# ruff: noqa: UP006 UP007 UP045
# @omlish-lite
import dataclasses as dc
import typing as ta


##


@dc.dataclass(frozen=True)
class SystemdListUnit:
    unit: str
    load: str  # loaded, not-found
    active: str  # active, inactive
    sub: str  # running, exited, dead
    description: str

    @classmethod
    def parse(cls, s: str) -> 'SystemdListUnit':
        return SystemdListUnit(*[p.strip() for p in s.strip().split(None, 4)])

    @classmethod
    def parse_all(cls, s: str) -> ta.List['SystemdListUnit']:
        return [
            cls.parse(sl)
            for l in s.strip().splitlines()
            if (sl := l.strip())
        ]


PARSABLE_SYSTEMD_LIST_UNIT_ARGS: ta.Sequence[str] = [
    '--all',
    '--no-legend',
    '--no-pager',
    '--plain',
]


##


def parse_systemd_show_output(s: str) -> ta.Mapping[str, str]:
    d: ta.Dict[str, str] = {}
    for l in s.strip().splitlines():
        if not (l := l.strip()):
            continue
        k, _, v = l.partition('=')
        d[k] = v
    return d



# ruff: noqa: UP006 UP007 UP045
import abc
import typing as ta


K = ta.TypeVar('K')
V = ta.TypeVar('V')


class KeyedCollectionAccessors(abc.ABC, ta.Generic[K, V]):
    @property
    @abc.abstractmethod
    def _by_key(self) -> ta.Mapping[K, V]:
        raise NotImplementedError

    def __iter__(self) -> ta.Iterator[V]:
        return iter(self._by_key.values())

    def __len__(self) -> int:
        return len(self._by_key)

    def __contains__(self, key: K) -> bool:
        return key in self._by_key

    def __getitem__(self, key: K) -> V:
        return self._by_key[key]

    def get(self, key: K, default: ta.Optional[V] = None) -> ta.Optional[V]:
        return self._by_key.get(key, default)

    def items(self) -> ta.Iterator[ta.Tuple[K, V]]:
        return iter(self._by_key.items())


class KeyedCollection(KeyedCollectionAccessors[K, V]):
    def __init__(self, items: ta.Iterable[V]) -> None:
        super().__init__()

        by_key: ta.Dict[K, V] = {}
        for v in items:
            if (k := self._key(v)) in by_key:
                raise KeyError(f'key {k} of {v} already registered by {by_key[k]}')
            by_key[k] = v
        self.__by_key = by_key

    @property
    def _by_key(self) -> ta.Mapping[K, V]:
        return self.__by_key

    @abc.abstractmethod
    def _key(self, v: V) -> K:
        raise NotImplementedError
